/***************************************************************************
                          cxferqueue.h  -  description
                             -------------------
    begin                : Don Mai 16 2002
    copyright            : (C) 2002-2005 by Mathias Kster
    email                : mathen@users.berlios.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef CXFERQUEUE_H
#define CXFERQUEUE_H

/**
  *@author Mathias Kster
  */

#include <time.h>

#include <dclib/dcos.h>
#include <dclib/dcobject.h>
#include <dclib/core/cthread.h>
#include <dclib/core/cobject.h>
#include <dclib/core/cstring.h>
#include <dclib/core/cstringlist.h>

class DLL_EXPORT DCTransferNick : public CObject {
public:
	/** */
	DCTransferNick() {
		m_nID = 0;
		m_eState = etwsIDLE;
		m_tTimeout = 0;
		m_tTimestamp = 0;
	};

	/** */
	virtual ~DCTransferNick() {};

	/** */
	ulonglong m_nID;
	/** */
	CString m_sNick;
	/** */
	CString m_sHubHost;
	/** */
	CString m_sHubName;
	/** */
	time_t m_tTimestamp;

	/** */
	time_t m_tTimeout;
	/** */
	eTransferWaitState m_eState;
};

class DLL_EXPORT DCTransferRemoteUsrFile : public CObject {
public:
	/** */
	DCTransferRemoteUsrFile( DCTransferRemoteUsrFile * p = 0 ) {
		if ( !p ) {
			m_nID = 0;
			m_nNickID = 0;
			m_nRemoteFileID = 0;
			m_eState = etfsNONE;
			m_bTTHL = FALSE;
		} else {
			m_nID = p->m_nID;
			m_nNickID = p->m_nNickID;
			m_nRemoteFileID = p->m_nRemoteFileID;
			m_eState = p->m_eState;
			m_bTTHL = p->m_bTTHL;
		}
	};

	/** */
	virtual ~DCTransferRemoteUsrFile() {};

	/** */
	ulonglong m_nID;
	/** */
	ulonglong m_nNickID;
	/** */
	ulonglong m_nRemoteFileID;
	/** */
	eTransferFileState m_eState;
	/** */
	bool m_bTTHL;
};

class DLL_EXPORT DCTransferRemoteFile : public CObject {
public:
	/** */
	DCTransferRemoteFile() {
		m_nID = 0;
		m_nLocalFileID = 0;
		m_nSize = 0;
		m_nPriority = 0;
		m_tTimestamp = 0;
	};

	/** */
	virtual ~DCTransferRemoteFile() {};

	/** */
	ulonglong m_nID;
	/** */
	ulonglong m_nLocalFileID;
	/** */
	CString m_sName;
	/** */
	CString m_sHash;
	/** */
	ulonglong m_nSize;
	/** */
	unsigned char m_nPriority;
	/** */
	time_t m_tTimestamp;
};

class DLL_EXPORT DCTransferChunk : public CObject {
public:
	/** */
	DCTransferChunk() {
		m_nID = 0;
		m_eState = ecsMISSING;
		m_nSize = 0;
	};
	/** */
	virtual ~DCTransferChunk() {};
	
	/** */
	ulonglong m_nID;
	/** */
	eChunkState m_eState;
	/** */
	ulonglong m_nSize;
	/** */
	CString m_sHash;
};

#ifdef WIN32
template class DLL_EXPORT CList<DCTransferChunk>;
#endif

class DLL_EXPORT DCTransferLocalFile : public CObject {
public:
	/** */
	DCTransferLocalFile() {
		m_nID = 0;
		m_nSize = 0;
		m_bMultiSource = FALSE;
		m_eState = etfsNONE;
		m_eMedium = eltNONE;
		m_nRefCount = 0;
		m_bTTHL = FALSE;
		m_tTimestamp = 0;
	};

	/** */
	virtual ~DCTransferLocalFile() {};

	/** */
	ulonglong m_nID;
	/** */
	eTransferFileState m_eState;
	/** */
	CList<DCTransferChunk> m_ChunkList;
	/** */
	CString m_sName;
	/** */
	CString m_sHash;
	/** */
	ulonglong m_nSize;
	/** */
	bool m_bMultiSource;
	/** file or buffer or client version */
	eltMedium m_eMedium;
	/** */
	ulonglong m_nRefCount;
	/** */
	bool m_bTTHL;
	/** */
	time_t m_tTimestamp;
};

class CByteArray;

class DLL_EXPORT CXFERQueue : public CObject {
public:
	/** */
	CXFERQueue();
	/** */
	virtual ~CXFERQueue();

	CMutex m_Mutex;

	/** */
	void Load();
	/** */
	void Save(time_t ttimeout = 0);
	
	/** */
	CThreadStringList * m_pTransferNickList;
	/** */
	ulonglong GetNickID( CString nick, CString hubhost );
	/** */
	bool SetNickXFER( CString nick, CString hubhost, CString hubname, eTransferWaitState estate );
	/** */
	DCTransferNick * GetNick( ulonglong nickid );
	/** */
	bool AddNick( CString nick, CString hubhost, CString hubname, ulonglong & id );

	/** */
	CThreadStringList * m_pTransferRemoteUsrFileList;
	/** */
	DCTransferRemoteUsrFile * GetRemoteUsrFile( ulonglong rusrfileid );
	/** */
	ulonglong AddRemoteUsrFile( ulonglong nickid, ulonglong rfileid );
	/** */
	void DelRemoteUsrFile( ulonglong rusrfileid );
	/** */
	void UpdateRemoteUsrFile( ulonglong rusrfileid, eTransferWaitState ustate, eTransferFileState rusrfstate, eTransferFileState lfstate );
	
	/** */
	CThreadStringList * m_pTransferRemoteFileList;
	/** */
	DCTransferRemoteFile * GetRemoteFile( ulonglong nickid, CString name );
	/** */
	DCTransferRemoteFile * GetRemoteFile( ulonglong rfileid );
	/** */
	ulonglong AddRemoteFile( ulonglong nickid, ulonglong lfileid, CString name, ulonglong size, CString hash, unsigned char priority );
	
	/** */
	bool GetNextXFER( CString nick, CString hubhost, DCTransferRemoteUsrFile *& rusrfile, DCTransferRemoteFile *& rfile, DCTransferLocalFile *& lfile );
	
	/** */
	CThreadStringList * m_pTransferLocalFileList;
	/** */
	DCTransferLocalFile * GetLocalFile( ulonglong lfileid );
	/** */
	ulonglong AddLocalFile( CString name, ulonglong size, CString hash, eltMedium medium, bool force = FALSE );
	/** */
	bool GetLocalFileChunk( DCTransferLocalFile * lfile, ulonglong * start, ulonglong * end );
	/** */
	bool SetLocalFileChunk( DCTransferLocalFile * lfile, ulonglong start, ulonglong end );
	/** */
	bool CheckFreeLocalFileChunk( DCTransferLocalFile * lfile );
	/** */
	bool CheckMissingFiles( ulonglong nickid );

	/** */
	bool VerfifyTTHL( ulonglong rusrfileid, CByteArray * ba );
	
	/** */
	void Update( time_t ttimeout );

private:
	/** */
	time_t m_tSaveTimeout;
	/** */
	time_t m_tUpdateTimeout;
};

#endif
